@@ -649,7 +649,7 @@ public class ZooKeeperMain {
             System.exit(0);
         } else if (cmd.equals("redo") && args.length >= 2) {
             Integer i = Integer.decode(args[1]);
-            if (commandCount <= i){ // don't allow redoing this redo
+            if (commandCount <= i || i < 0){ // don't allow redoing this redo
                 System.out.println("Command index out of range");
                 return false;
             }
